=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-11-28 01:56:31 +0000
+++ bzrlib/errors.py	2006-12-07 23:18:34 +0000
@@ -932,11 +932,12 @@
 
 class KnitHeaderError(KnitError):
 
-    _fmt = "Knit header error: %(badline)r unexpected"
+    _fmt = "Knit header error: %(badline)r unexpected for file %(filename)s"
 
-    def __init__(self, badline):
+    def __init__(self, badline, filename):
         KnitError.__init__(self)
         self.badline = badline
+        self.filename = filename
 
 
 class KnitCorrupt(KnitError):

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-12-07 00:39:14 +0000
+++ bzrlib/knit.py	2006-12-07 23:18:34 +0000
@@ -980,8 +980,13 @@
 
     def check_header(self, fp):
         line = fp.readline()
+        if line == '':
+            # An empty file can actually be treated as though the file doesn't
+            # exist yet.
+            raise errors.NoSuchFile(self._transport.base + self._filename)
         if line != self.HEADER:
-            raise KnitHeaderError(badline=line)
+            raise KnitHeaderError(badline=line,
+                              filename=self._transport.abspath(self._filename))
 
     def commit(self):
         """Commit is a nop."""

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2006-11-10 22:23:15 +0000
+++ bzrlib/tests/test_errors.py	2006-12-07 23:18:34 +0000
@@ -46,6 +46,11 @@
         error = errors.InstallFailed([None])
         self.assertEqual("Could not install revisions:\nNone", str(error))
 
+    def test_knit_header_error(self):
+        error = errors.KnitHeaderError('line foo\n', 'path/to/file')
+        self.assertEqual("Knit header error: 'line foo\\n' unexpected"
+                         " for file path/to/file", str(error))
+
     def test_medium_not_connected(self):
         error = errors.MediumNotConnected("a medium")
         self.assertEqualDiff(

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2006-11-28 15:11:37 +0000
+++ bzrlib/tests/test_knit.py	2006-12-07 23:18:34 +0000
@@ -20,6 +20,7 @@
 import difflib
 
 
+from bzrlib import errors
 from bzrlib.errors import KnitError, RevisionAlreadyPresent, NoSuchFile
 from bzrlib.knit import (
     KnitContent,
@@ -85,6 +86,7 @@
         self.assertEqual(it.next(), (1, 2, 2, [("", "a"), ("", "c")]))
         self.assertRaises(StopIteration, it.next)
 
+
 class KnitTests(TestCaseWithTransport):
     """Class containing knit test helper routines."""
 
@@ -746,3 +748,19 @@
         # And it shouldn't be modified
         self.assertEqual(['a-1'], idx._history)
         self.assertEqual({'a-1':('a-1', ['fulltext'], 0, 0, [], 0)}, idx._cache)
+
+    def test_knit_index_ignores_empty_files(self):
+        # There was a race condition in older bzr, where a ^C at the right time
+        # could leave an empty .kndx file, which bzr would later claim was a
+        # corrupted file since the header was not present. In reality, the file
+        # just wasn't created, so it should be ignored.
+        t = get_transport('.')
+        t.put_bytes('test.kndx', '')
+
+        knit = self.make_test_knit()
+
+    def test_knit_index_checks_header(self):
+        t = get_transport('.')
+        t.put_bytes('test.kndx', '# not really a knit header\n\n')
+
+        self.assertRaises(errors.KnitHeaderError, self.make_test_knit)

